MODULE PRECISION_MODEL
   IMPLICIT NONE
   INTEGER, PARAMETER, PUBLIC   :: STND = SELECTED_REAL_KIND(12, 60)
END MODULE PRECISION_MODEL

MODULE MVBRVMOD
USE PRECISION_MODEL

CONTAINS
     SUBROUTINE MVKBRV( N, MINVLS, MAXVLS, NF, FUNCTN, ABSEPS, RELEPS, &
                                        ABSERR, FINEST, INTVLS, INFORM )
!
!  Automatic Multidimensional Integration Subroutine
!               
!       Alan Genz
!       Department of Mathematics
!       Washington State University
!       Pullman, Wa 99164-3113
!       Email : alangenz@wsu.edu
!
!         Last Change: 7/01
!
!  MVKBRV computes an approximation to the integral
!
!      1  1     1
!     I  I ... I   F(X)  dx(N)...dx(2)dx(1)
!      0  0     0
!
!    F(X) is an NF-vector of real integrands.
!
!  MVKBRV uses randomized Korobov rules. 
!  The primary references are
!   "Randomization of Number Theoretic Methods for Multiple Integration"
!    R. Cranley and T.N.L. Patterson, SIAM J Numer Anal, 13, pp. 904-14,
!  and 
!   "Optimal Parameters for Multidimensional Integration", 
!    P. Keast, SIAM J Numer Anal, 10, pp.831-838.
!  If N > 100 the subroutine uses quasi-random Richtmeyer points for 
!  variables with indices > 100. A reference is
!   "Methods of Numerical Integration", P.J. Davis and P. Rabinowitz, 
!    Academic Press, 1984, pp. 482-483.
!
!   
!**************  Parameters ********************************************
!***** Input parameters
!  N       Number of variables, must exceed 1, but not exceed 100
!  MINVLS  Integer minimum number of function evaluations allowed.
!          MINVLS must not exceed MAXVLS.  If MINVLS < 0 then the
!          routine assumes a previous call has been made with 
!          the same integrand and continues that calculation.
!  MAXVLS  Integer maximum number of function evaluations allowed.
!  NF      Integer number of integrands, must exceed 1 and not exceed 5000
!  FUNCTN  user defined function to be integrated.
!          It must have parameters (N,Z), where Z is a real array
!          of dimension N.
!                                     
!  ABSEPS  Required absolute accuracy.
!  RELEPS  Required relative accuracy.
!***** Output parameters
!  MINVLS  Actual number of function evaluations used.
!  ABSERR  Estimated absolute accuracy for FINEST(1).
!  FINEST  Estimated NF-vector of values of integrals.
!  INFORM  INFORM = 0 for normal exit, when 
!                     ABSERR <= MAX(ABSEPS, RELEPS*ABS(FINEST(1)))
!                  and 
!                     INTVLS <= MAXCLS.
!          INFORM = 1 If MAXVLS was too small to obtain the required 
!          accuracy. In this case a value FINEST is returned with 
!          estimated absolute accuracy ABSERR.
!***********************************************************************
!
! Global Variables
!
      INTEGER,                             INTENT(IN) :: N, NF, MINVLS, MAXVLS 
      INTERFACE 
         FUNCTION FUNCTN( NF, X ) RESULT(VALUE)
            USE PRECISION_MODEL
            INTEGER,         INTENT(IN)                :: NF
            REAL(KIND=STND), INTENT(IN), DIMENSION(:)  :: X
            REAL(KIND=STND),             DIMENSION(NF) :: VALUE
         END FUNCTION FUNCTN
      END INTERFACE   
      REAL(KIND=STND),                    INTENT(IN)  :: ABSEPS, RELEPS
      REAL(KIND=STND),     DIMENSION(NF), INTENT(OUT) :: FINEST
      REAL(KIND=STND),                    INTENT(OUT) :: ABSERR
      INTEGER,                            INTENT(OUT) :: INTVLS, INFORM
!
! Local Variables  
!

      INTEGER,                   PARAMETER :: PL = 28, NM = 99, NMP = NM + 1
      INTEGER,                   PARAMETER :: FM = 5000, NMX = 1000
      INTEGER,                   PARAMETER :: MINSMP = 8
      REAL(KIND=STND),           PARAMETER :: ONE = 1
      INTEGER,                        SAVE :: NP, SAMPLS
      REAL(KIND=STND), DIMENSION(FM), SAVE :: VAREST
      INTEGER,      DIMENSION(PL,NM)       :: C
      INTEGER                              :: I, K, KMX
      REAL(KIND=STND)                      :: VARPRD
      REAL(KIND=STND),       DIMENSION(NF) :: DIFINT, FINVAL, VARSQR
      REAL(KIND=STND),        DIMENSION(N) :: VK
      INTEGER, DIMENSION(PL), PARAMETER :: P = (/  31,      47,     73,  113, &
          173,    263,    397,    593,    907,   1361,    2053,   3079, 4621, &
         6947,  10427,  15641,  23473,  35221,  52837,   79259, 118891,       &
       178349, 267523, 401287, 601942, 902933, 1354471, 2031713 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C01 = (/     12,      9,      9, &
                12,     12,     12,     12,     12,     12,     12,     12, &
                12,      3,      3,      3,     12,      7,      7,     12, &
                12,     12,     12,     12,     12,     12,     12,     12, &
                 3,      3,      3,     12,      7,      7,     12,     12, &
                12,     12,     12,     12,     12,     12,     12,      3, &
                 3,      3,     12,      7,      7,     12,     12,     12, &
                12,     12,     12,     12,     12,     12,      3,      3, &
                 3,     12,      7,      7,     12,     12,     12,     12, &
                12,     12,     12,     12,      7,      3,      3,      3, &
                 7,      7,      7,      3,      3,      3,      3,      3, &
                 3,      3,      3,      3,      3,      3,      3,      3, &
                 3,      3,      3,      3,      3,      3,      3,      3 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C02 = (/     13,     11,     11, &
                10,     15,     15,     15,     15,     22,     15,     15, &
                15,     15,      6,      6,      6,     15,     15,      9, &
                13,      2,      2,      2,     13,     11,     11,     10, &
                15,     15,     15,     15,     15,     15,     15,     15, &
                15,      6,      6,      6,     15,     15,      9,     13, &
                 2,      2,      2,     13,     11,     11,     10,     15, &
                15,     15,     15,     15,     15,     15,     15,     15, &
                 6,      6,      6,     15,     15,      9,     13,      2, &
                 2,      2,     13,     11,     11,     10,     10,     15, &
                15,     15,     15,     15,     15,     15,     15,      6, &
                 2,      3,      2,      3,      2,      2,      2,      2, &
                 2,      2,      2,      2,      2,      2,      2,      2 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C03 = (/     27,     28,     10, &
                11,     11,     20,     11,     20,     28,     13,     13, &
                28,     13,     13,     13,     14,     14,     14,     14, &
                14,     14,     14,     14,     14,     14,     14,     14, &
                14,     14,     14,     14,     31,     31,      5,      5, &
                 5,     31,     13,     11,     11,     11,     11,     11, &
                11,     13,     13,     13,     13,     13,     13,     13, &
                14,     14,     14,     14,     14,     14,     14,     14, &
                14,     14,     14,     14,     14,     14,     14,     14, &
                31,     31,      5,      5,      5,     11,     13,     11, &
                11,     11,     11,     11,     11,     11,     13,     13, &
                11,     13,      5,      5,      5,      5,     14,     13, &
                 5,      5,      5,      5,      5,      5,      5,      5 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C04 = (/     35,     27,     27, &
                36,     22,     29,     29,     20,     45,      5,      5, &
                 5,     21,     21,     21,     21,     21,     21,     21, &
                21,     21,     21,     21,     21,     21,     21,     21, &
                21,     29,     17,     17,     17,     17,     17,     17, &
                17,     17,     17,     17,     23,     23,     23,     23, &
                23,     23,     23,     23,     23,     23,     23,     23, &
                21,     27,      3,      3,      3,     24,     27,     27, &
                17,     29,     29,     29,     17,      5,      5,      5, &
                 5,     21,     21,     21,     21,     21,     21,     21, &
                21,     21,     21,     21,     21,     21,     21,     21, &
                21,     17,     17,     17,      6,     17,     17,      6, &
                 3,      6,      6,      3,      3,      3,      3,      3 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C05 = (/     64,     76,     28, &
                28,     44,     44,     55,     31,     10,     52,     10, &
                52,     10,     10,     38,     38,     10,     52,     10, &
                10,     10,     49,     60,     49,     49,     49,     49, &
                49,     49,     49,     49,     49,     49,     38,     38, &
                31,      4,      4,     31,     64,      4,      4,      4, &
                64,     45,     45,     45,     45,     45,     45,     66, &
                66,     66,     66,     66,     66,     66,     66,     66, &
                66,     66,     66,     66,     66,     66,     66,     66, &
                66,     66,     11,     66,     66,     66,     66,     66, &
                66,     66,     66,     66,     45,     11,      7,      3, &
                 2,      2,      2,     27,      5,      3,      3,      5, &
                 5,      2,      2,      2,      2,      2,      2,      2 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C06 = (/    111,     42,     54, &
               118,     20,     31,     31,     72,     17,     94,     14, &
                14,     11,     14,     14,     14,     94,     10,     10, &
                10,     10,     14,     14,     14,     14,     14,     14, &
                14,     11,     11,     11,      8,      8,      8,      8, &
                 8,      8,      8,     18,     18,     18,     18,     18, &
               113,     62,     62,     45,     45,    113,    113,    113, &
               113,    113,    113,    113,    113,    113,    113,    113, &
               113,    113,    113,    113,    113,    113,     63,     63, &
                53,     63,     67,     67,     67,     67,     67,     67, &
                67,     67,     67,     67,     67,     67,     67,     67, &
                67,     51,     51,     51,     51,     51,     12,     51, &
                12,     51,      5,      3,      3,      2,      2,      5 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C07 = (/    163,    154,     83, &
                43,     82,     92,    150,     59,     76,     76,     47, &
                11,     11,    100,    131,    116,    116,    116,    116, &
               116,    116,    138,    138,    138,    138,    138,    138, &
               138,    138,    138,    101,    101,    101,    101,    101, &
               101,    101,    101,    101,    101,    101,    101,    101, &
               101,    101,    101,    101,    101,    101,    101,    101, &
               116,    116,    116,    116,    116,    116,    100,    100, &
               100,    100,    100,    138,    138,    138,    138,    138, &
               101,    101,    101,    101,    101,    101,    101,    101, &
               101,    101,    101,    101,    101,    101,    101,    101, &
               101,    101,    101,     38,     38,     38,     38,     38, &
                38,     38,     38,      3,      3,      3,      3,      3 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C08 = (/    246,    189,    242, &
               250,    250,    250,    102,    250,     36,    118,    196, &
               118,    191,    215,     49,    121,     49,    121,     49, &
                49,     49,     49,    121,     49,     49,     49,     49, &
                49,    171,    171,    171,    171,    171,    171,    171, &
               171,    171,    171,    171,    171,    171,    171,    171, &
               171,    171,    171,    171,    171,    171,    171,    171, &
               171,    171,    171,    171,    171,    171,    171,    171, &
               171,    171,    171,    161,    161,    161,    161,    161, &
               161,    161,    161,     14,     14,     14,     14,     14, &
                14,     14,     14,     14,     14,     14,     14,     14, &
                14,     14,     14,     14,     10,     10,     10,     10, &
                10,     10,    103,     10,     10,     10,     10,      5 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C09 = (/    347,    402,    322, &
               418,    215,    220,    339,    339,    339,    337,    218, &
               167,    315,    315,    315,    167,    167,    167,    167, &
               361,    201,    124,    124,    124,    124,    124,    124, &
               124,    124,    124,    124,    124,    231,    231,     90, &
                90,     90,     90,     90,     90,     90,     90,     90, &
                90,     90,     90,     90,     90,     48,     48,     48, &
                48,     90,     90,     90,     90,     90,     90,     90, &
                90,     90,     90,     90,     90,     90,     90,     90, &
                90,     90,     90,     90,     90,     90,     90,     90, &
               243,    243,    243,    243,    243,    243,    243,    243, &
               243,    243,    283,    283,    283,    283,    283,    283, &
               283,    283,    283,     16,    283,     16,    283,    283 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C10 = (/    505,    220,    601, &
               644,    612,    160,    206,    206,    206,    422,    134, &
               518,    134,    134,    518,    652,    382,    206,    158, &
               441,    179,    441,     56,    559,    559,     56,     56, &
                56,     56,     56,     56,     56,     56,     56,     56, &
                56,     56,     56,     56,    101,    101,     56,    101, &
               101,    101,    101,    101,    101,    101,    101,    193, &
               193,    193,    193,    193,    193,    193,    101,    101, &
               101,    101,    101,    101,    101,    101,    101,    101, &
               101,    101,    101,    101,    101,    101,    101,    101, &
               101,    101,    101,    122,    122,    122,    122,    122, &
               122,    122,    122,    122,    122,    122,    122,    122, &
               122,    122,    122,    122,    101,    101,    101,    101 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C11 = (/    794,    325,    960, &
               528,    247,    247,    338,    366,    847,    753,    753, &
               236,    334,    334,    461,    711,    652,    381,    381, &
               381,    652,    381,    381,    381,    381,    381,    381, &
               381,    226,    326,    326,    326,    326,    326,    326, &
               326,    126,    326,    326,    326,    326,    326,    326, &
               326,    326,    326,    326,    195,    195,     55,     55, &
                55,     55,     55,     55,     55,     55,     55,     55, &
                55,     55,     55,     55,     55,     55,     55,     55, &
                55,    195,    195,    195,    195,    195,    195,    195, &
               132,    132,    132,    132,    132,    132,    132,    132, &
               132,    132,    132,    387,    387,    387,    387,    387, &
               387,    387,    387,    387,    387,    387,    387,    387 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C12 = (/   1189,    888,    259, &
              1082,    725,    811,    636,    965,    497,    497,   1490, &
              1490,    392,   1291,    508,    508,   1291,   1291,    508, &
              1291,    508,    508,    867,    867,    934,    867,    934, &
               867,    867,    867,    867,    867,    867,    867,   1284, &
              1284,   1284,   1284,   1284,   1284,   1284,   1284,   1284, &
               563,    563,    563,    563,   1010,   1010,   1010,    208, &
               838,    563,    563,    563,    759,    759,    564,    759, &
               759,    801,    801,    801,    801,    759,    759,    759, &
               759,    759,    563,    563,    563,    563,    563,    563, &
               563,    563,    226,    226,    226,    226,    226,    226, &
               226,    226,    226,    226,    226,    226,    226,    226, &
               226,    226,    226,    226,    226,    226,    226,    226 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C13 = (/   1763,   1018,   1500, &
               432,   1332,   1159,    126,   2240,   1719,   1284,    878, &
              1983,    266,    266,    266,    266,    747,    747,    127, &
               127,   2074,    127,   2074,   1400,   1383,   1383,   1383, &
              1383,   1383,   1383,   1383,   1383,   1383,   1383,   1400, &
              1383,   1383,   1383,   1383,   1383,   1383,   1383,    507, &
              1073,   1073,   1073,   1073,   1990,   1990,   1990,   1990, &
              1990,    507,    507,    507,    507,    507,    507,    507, &
               507,    507,   1073,   1073,   1073,   1073,   1073,   1073, &
              1073,   1073,   1073,   1073,   1073,   1073,   1073,   1073, &
              1073,   1073,   1073,     22,     22,     22,     22,     22, &
                22,   1073,    452,    452,    452,    452,    452,    452, &
               318,    301,    301,    301,    301,     86,     86,     15 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C14 = (/   2872,   3233,   1534, &
              2941,   2910,    393,   1796,    919,    446,    919,    919, &
              1117,    103,    103,    103,    103,    103,    103,    103, &
              2311,   3117,   1101,   3117,   3117,   1101,   1101,   1101, &
              1101,   1101,   2503,   2503,   2503,   2503,   2503,   2503, &
              2503,   2503,    429,    429,    429,    429,    429,    429, &
               429,   1702,   1702,   1702,    184,    184,    184,    184, &
               184,    105,    105,    105,    105,    105,    105,    105, &
               105,    105,    105,    105,    105,    105,    105,    105, &
               105,    105,    105,    105,    105,    105,    105,    105, &
               105,    105,    105,    105,    105,    105,    105,    105, &
               105,    105,    105,    784,    784,    784,    784,    784, &
               784,    784,    784,    784,    784,    784,    784,    784 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C15 = (/   4309,   3758,   4034, &
              1963,    730,    642,   1502,   2246,   3834,   1511,   1102, &
              1102,   1522,   1522,   3427,   3427,   3928,    915,    915, &
              3818,   3818,   3818,   3818,   4782,   4782,   4782,   3818, &
              4782,   3818,   3818,   1327,   1327,   1327,   1327,   1327, &
              1327,   1327,   1387,   1387,   1387,   1387,   1387,   1387, &
              1387,   1387,   1387,   2339,   2339,   2339,   2339,   2339, &
              2339,   2339,   2339,   2339,   2339,   2339,   2339,   2339, &
              3148,   3148,   3148,   3148,   3148,   3148,   3148,   3148, &
              3148,   3148,   3148,   3148,   3148,   3148,   3148,   3148, &
              3148,   3148,   1776,   1776,   1776,   3354,   3354,   3354, &
               925,   3354,   3354,    925,    925,    925,    925,    925, &
              2133,   2133,   2133,   2133,   2133,   2133,   2133,   2133 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C16 = (/   6610,   6977,   1686, &
              3819,   2314,   5647,   3953,   3614,   5115,    423,    423, &
              5408,   7652,    423,    423,    487,   6227,   2660,   6227, &
              1221,   3811,    197,   4367,   4367,   1281,   1221,    351, &
               351,    351,   1984,   1984,   3995,   2999,   2999,   2999, &
              2999,   2999,   2999,   2063,   2063,   2063,   2063,   1644, &
              2063,   2077,   2512,   2512,   2512,   2077,   2077,   2077, &
              2077,    754,    754,    754,    754,    754,    754,    754, &
               754,    754,    754,    754,    754,    754,    754,    754, &
               754,    754,    754,    754,   1097,   1097,    754,    754, &
               754,    754,    248,    754,   1097,   1097,   1097,   1097, &
               222,    222,    222,    222,    754,   1982,   1982,   1982, &
              1982,   1982,   1982,   1982,   1982,   1982,   1982,   1982 /)  
      INTEGER, DIMENSION(NM), PARAMETER :: C17 = (/   9861,   3647,   4073, &
              2535,   3430,   9865,   2830,   9328,   4320,   5913,  10365, &
              8272,   3706,   6186,   7806,   7806,   7806,   8610,   2563, &
             11558,  11558,   9421,   1181,   9421,   1181,   1181,   1181, &
              9421,   1181,   1181,  10574,  10574,   3534,   3534,   3534, &
              3534,   3534,   2898,   2898,   2898,   3450,   2141,   2141, &
              2141,   2141,   2141,   2141,   2141,   7055,   7055,   7055, &
              7055,   7055,   7055,   7055,   7055,   7055,   7055,   7055, &
              7055,   7055,   7055,   7055,   2831,   8204,   8204,   8204, &
              8204,   8204,   8204,   8204,   8204,   8204,   8204,   8204, &
              8204,   8204,   8204,   8204,   8204,   8204,   8204,   8204, &
              8204,   8204,   8204,   8204,   8204,   4688,   4688,   4688, &
              2831,   2831,   2831,   2831,   2831,   2831,   2831,   2831 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C18 = (/  10327,   7582,   7124, &
              8214,   9600,  10271,  10193,  10800,   9086,   2365,   4409, &
             13812,   5661,   9344,   9344,  10362,   9344,   9344,   8585, &
             11114,  13080,  13080,  13080,   6949,   3436,   3436,   3436, &
             13213,   6130,   6130,   8159,   8159,  11595,   8159,   3436, &
              7096,   7096,   7096,   7096,   7096,   7096,   7096,   7096, &
              7096,   7096,   7096,   7096,   7096,   7096,   7096,   7096, &
              7096,   7096,   4377,   7096,   4377,   4377,   4377,   4377, &
              4377,   5410,   5410,   4377,   4377,   4377,   4377,   4377, &
              4377,   4377,   4377,   4377,   4377,   4377,   4377,   4377, &
              4377,   4377,   4377,   4377,   4377,   4377,   4377,   4377, &
              4377,   4377,   4377,   4377,   4377,   4377,   4377,   4377, &
              4377,   4377,   4377,    440,    440,   1199,   1199,   1199 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C19 = (/  19540,  19926,  11582, &
             11113,  24585,   8726,  17218,    419,   4918,   4918,   4918, &
             15701,  17710,   4037,   4037,  15808,  11401,  19398,  25950, &
             25950,   4454,  24987,  11719,   8697,   1452,   1452,   1452, &
              1452,   1452,   8697,   8697,   6436,  21475,   6436,  22913, &
              6434,  18497,  11089,  11089,  11089,  11089,   3036,   3036, &
             14208,  14208,  14208,  14208,  12906,  12906,  12906,  12906, &
             12906,  12906,  12906,  12906,   7614,   7614,   7614,   7614, &
              5021,   5021,   5021,   5021,   5021,   5021,  10145,  10145, &
             10145,  10145,  10145,  10145,  10145,  10145,  10145,  10145, &
             10145,  10145,  10145,  10145,  10145,  10145,  10145,  10145, &
             10145,  10145,  10145,  10145,  10145,  10145,   4544,   4544, &
              4544,   4544,   4544,   4544,   8394,   8394,   8394,   8394 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C20 = (/  34566,   9579,  12654, &
             26856,  37873,  38806,  29501,  17271,   3663,  10763,  18955, &
              1298,  26560,  17132,  17132,   4753,   4753,   8713,  18624, &
             13082,   6791,   1122,  19363,  34695,  18770,  18770,  18770, &
             18770,  15628,  18770,  18770,  18770,  18770,  33766,  20837, &
             20837,  20837,  20837,  20837,  20837,   6545,   6545,   6545, &
              6545,   6545,  12138,  12138,  12138,  12138,  12138,  12138, &
             12138,  12138,  12138,  12138,  12138,  12138,  12138,  12138, &
             30483,  30483,  30483,  30483,  30483,  12138,  12138,  12138, &
             12138,  12138,  12138,  12138,  12138,  12138,  12138,  12138, &
             12138,  12138,  12138,  12138,  12138,  12138,  12138,  12138, &
              9305,  11107,  11107,  11107,  11107,  11107,  11107,  11107, &
             11107,  11107,  11107,  11107,  11107,  11107,   9305,   9305 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C21 = (/  31929,  49367,  10982, &
              3527,  27066,  13226,  56010,  18911,  40574,  20767,  20767, &
              9686,  47603,  47603,  11736,  11736,  41601,  12888,  32948, &
             30801,  44243,  53351,  53351,  16016,  35086,  35086,  32581, &
              2464,   2464,  49554,   2464,   2464,  49554,  49554,   2464, &
                81,  27260,  10681,   2185,   2185,   2185,   2185,   2185, &
              2185,   2185,  18086,  18086,  18086,  18086,  18086,  17631, &
             17631,  18086,  18086,  18086,  37335,  37774,  37774,  37774, &
             26401,  26401,  26401,  26401,  26401,  26401,  26401,  26401, &
             26401,  26401,  26401,  26401,  26401,  12982,  40398,  40398, &
             40398,  40398,  40398,  40398,   3518,   3518,   3518,  37799, &
             37799,  37799,  37799,  37799,  37799,  37799,  37799,  37799, &
              4721,   4721,   4721,   4721,   7067,   7067,   7067,   7067 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C22 = (/  40701,  69087,  77576, &
             64590,  39397,  33179,  10858,  38935,  43129,  35468,  35468, &
              5279,  61518,  61518,  27945,  70975,  70975,  86478,  86478, &
             20514,  20514,  73178,  73178,  43098,  43098,   4701,  59979, &
             59979,  58556,  69916,  15170,  15170,   4832,   4832,  43064, &
             71685,   4832,  15170,  15170,  15170,  27679,  27679,  27679, &
             60826,  60826,   6187,   6187,   4264,   4264,   4264,   4264, &
              4264,  45567,  32269,  32269,  32269,  32269,  62060,  62060, &
             62060,  62060,  62060,  62060,  62060,  62060,  62060,   1803, &
              1803,   1803,   1803,   1803,   1803,   1803,   1803,   1803, &
              1803,   1803,   1803,   1803,  51108,  51108,  51108,  51108, &
             51108,  51108,  51108,  51108,  51108,  51108,  51108,  51108, &
             55315,  55315,  54140,  54140,  54140,  54140,  54140,  13134 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C23 = (/ 103650, 125480,  59978, &
             46875,  77172,  83021, 126904,  14541,  56299,  43636,  11655, &
             52680,  88549,  29804, 101894, 113675,  48040, 113675,  34987, &
             48308,  97926,   5475,  49449,   6850,  62545,  62545,   9440, &
             33242,   9440,  33242,   9440,  33242,   9440,  62850,   9440, &
              9440,   9440,  90308,  90308,  90308,  47904,  47904,  47904, &
             47904,  47904,  47904,  47904,  47904,  47904,  41143,  41143, &
             41143,  41143,  41143,  41143,  41143,  36114,  36114,  36114, &
             36114,  36114,  24997,  65162,  65162,  65162,  65162,  65162, &
             65162,  65162,  65162,  65162,  65162,  65162,  65162,  65162, &
             65162,  47650,  47650,  47650,  47650,  47650,  47650,  47650, &
             40586,  40586,  40586,  40586,  40586,  40586,  40586,  38725, &
             38725,  38725,  38725,  88329,  88329,  88329,  88329,  88329 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C24 = (/ 165843,  90647,  59925, &
            189541,  67647,  74795,  68365, 167485, 143918,  74912, 167289, &
             75517,   8148, 172106, 126159,  35867,  35867,  35867, 121694, &
             52171,  95354, 113969, 113969,  76304, 123709, 123709, 144615, &
            123709,  64958,  64958,  32377, 193002, 193002,  25023,  40017, &
            141605, 189165, 189165, 141605, 189165, 189165, 141605, 141605, &
            141605, 189165, 127047, 127047, 127047, 127047, 127047, 127047, &
            127047, 127047, 127047, 127047, 127047, 127047, 127047, 127047, &
            127047, 127047, 127047, 127047, 127047, 127047, 127785, 127785, &
            127785, 127785, 127785, 127785, 127785, 127785, 127785, 127785, &
             80822,  80822,  80822,  80822,  80822,  80822, 131661, 131661, &
            131661, 131661, 131661, 131661, 131661, 131661, 131661, 131661, &
            131661, 131661, 131661, 131661, 131661, 131661,   7114, 131661 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C25 = (/ 130365, 236711, 110235, &
            125699,  56483,  93735, 234469,  60549,   1291,  93937, 245291, &
            196061, 258647, 162489, 176631, 204895,  73353, 172319,  28881, &
            136787, 122081, 122081, 275993,  64673, 211587, 211587, 211587, &
            282859, 282859, 211587, 242821, 256865, 256865, 256865, 122203, &
            291915, 122203, 291915, 291915, 122203,  25639,  25639, 291803, &
            245397, 284047, 245397, 245397, 245397, 245397, 245397, 245397, &
            245397,  94241,  66575,  66575, 217673, 217673, 217673, 217673, &
            217673, 217673, 217673, 217673, 217673, 217673, 217673, 217673, &
            217673, 217673, 217673, 217673, 217673, 217673, 217673, 210249, &
            210249, 210249, 210249, 210249, 210249, 210249, 210249, 210249, &
            210249,  94453,  94453,  94453,  94453,  94453,  94453,  94453, &
             94453,  94453,  94453,  94453,  94453,  94453,  94453,  94453 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C26 = (/ 333459, 375354, 102417, &
            383544, 292630,  41147, 374614,  48032, 435453, 281493, 358168, &
            114121, 346892, 238990, 317313, 164158,  35497,  70530,  70530, &
            434839,  24754,  24754,  24754, 393656, 118711, 118711, 148227, &
            271087, 355831,  91034, 417029, 417029,  91034,  91034, 417029, &
             91034, 299843, 299843, 413548, 413548, 308300, 413548, 413548, &
            413548, 308300, 308300, 308300, 413548, 308300, 308300, 308300, &
            308300, 308300,  15311,  15311,  15311,  15311, 176255, 176255, &
             23613,  23613,  23613,  23613,  23613,  23613, 172210, 204328, &
            204328, 204328, 204328, 121626, 121626, 121626, 121626, 121626, &
            200187, 200187, 200187, 200187, 200187, 121551, 121551, 248492, &
            248492, 248492, 248492, 248492, 248492, 248492, 248492, 248492, &
            248492, 248492, 248492,  13942,  13942,  13942,  13942,  13942 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C27 = (/ 500884, 566009, 399251, &
            652979, 355008, 430235, 328722, 670680, 405585, 405585, 424646, &
            670180, 670180, 641587, 215580,  59048, 633320,  81010,  20789, &
            389250, 389250, 638764, 638764, 389250, 389250, 398094,  80846, &
            147776, 147776, 296177, 398094, 398094, 147776, 147776, 396313, &
            578233, 578233, 578233,  19482, 620706, 187095, 620706, 187095, &
            126467, 241663, 241663, 241663, 241663, 241663, 241663, 241663, &
            241663, 241663, 241663, 241663, 241663, 321632,  23210,  23210, &
            394484, 394484, 394484,  78101,  78101,  78101, 542095, 542095, &
            542095, 542095, 542095, 542095, 542095, 542095, 542095, 542095, &
            542095, 542095, 542095, 542095, 542095, 542095, 542095, 542095, &
            542095, 277743, 277743, 277743, 457259, 457259, 457259, 457259, &
            457259, 457259, 457259, 457259, 457259, 457259, 457259, 457259 /)
      INTEGER, DIMENSION(NM), PARAMETER :: C28 = (/ 858339, 918142, 501970, &
            234813, 460565,  31996, 753018, 256150, 199809, 993599, 245149, &
            794183, 121349, 150619, 376952, 809123, 809123, 804319,  67352, &
            969594, 434796, 969594, 804319, 391368, 761041, 754049, 466264, &
            754049, 754049, 466264, 754049, 754049, 282852, 429907, 390017, &
            276645, 994856, 250142, 144595, 907454, 689648, 687580, 687580, &
            687580, 687580, 978368, 687580, 552742, 105195, 942843, 768249, &
            307142, 307142, 307142, 307142, 880619, 880619, 880619, 880619, &
            880619, 880619, 880619, 117185, 117185, 117185, 117185, 117185, &
            117185, 117185, 117185, 117185, 117185, 117185,  60731,  60731, &
             60731,  60731,  60731,  60731,  60731,  60731,  60731,  60731, &
             60731, 178309, 178309, 178309, 178309,  74373,  74373,  74373, &
             74373,  74373,  74373,  74373,  74373, 214965, 214965, 214965 /)
      INTEGER, DIMENSION(NMX), PARAMETER ::   PRIME = (/            &
           2,    3,    5,    7,   11,   13,   17,   19,   23,   29, &
          31,   37,   41,   43,   47,   53,   59,   61,   67,   71, &
          73,   79,   83,   89,   97,  101,  103,  107,  109,  113, &
         127,  131,  137,  139,  149,  151,  157,  163,  167,  173, &
         179,  181,  191,  193,  197,  199,  211,  223,  227,  229, &
         233,  239,  241,  251,  257,  263,  269,  271,  277,  281, &
         283,  293,  307,  311,  313,  317,  331,  337,  347,  349, &
         353,  359,  367,  373,  379,  383,  389,  397,  401,  409, &
         419,  421,  431,  433,  439,  443,  449,  457,  461,  463, &
         467,  479,  487,  491,  499,  503,  509,  521,  523,  541, &
         547,  557,  563,  569,  571,  577,  587,  593,  599,  601, &
         607,  613,  617,  619,  631,  641,  643,  647,  653,  659, &
         661,  673,  677,  683,  691,  701,  709,  719,  727,  733, &
         739,  743,  751,  757,  761,  769,  773,  787,  797,  809, &
         811,  821,  823,  827,  829,  839,  853,  857,  859,  863, &
         877,  881,  883,  887,  907,  911,  919,  929,  937,  941, &
         947,  953,  967,  971,  977,  983,  991,  997, 1009, 1013, &
        1019, 1021, 1031, 1033, 1039, 1049, 1051, 1061, 1063, 1069, &
        1087, 1091, 1093, 1097, 1103, 1109, 1117, 1123, 1129, 1151, &
        1153, 1163, 1171, 1181, 1187, 1193, 1201, 1213, 1217, 1223, &
        1229, 1231, 1237, 1249, 1259, 1277, 1279, 1283, 1289, 1291, &
        1297, 1301, 1303, 1307, 1319, 1321, 1327, 1361, 1367, 1373, &
        1381, 1399, 1409, 1423, 1427, 1429, 1433, 1439, 1447, 1451, &
        1453, 1459, 1471, 1481, 1483, 1487, 1489, 1493, 1499, 1511, &
        1523, 1531, 1543, 1549, 1553, 1559, 1567, 1571, 1579, 1583, &
        1597, 1601, 1607, 1609, 1613, 1619, 1621, 1627, 1637, 1657, &
        1663, 1667, 1669, 1693, 1697, 1699, 1709, 1721, 1723, 1733, &
        1741, 1747, 1753, 1759, 1777, 1783, 1787, 1789, 1801, 1811, &
        1823, 1831, 1847, 1861, 1867, 1871, 1873, 1877, 1879, 1889, &
        1901, 1907, 1913, 1931, 1933, 1949, 1951, 1973, 1979, 1987, &
        1993, 1997, 1999, 2003, 2011, 2017, 2027, 2029, 2039, 2053, &
        2063, 2069, 2081, 2083, 2087, 2089, 2099, 2111, 2113, 2129, &
        2131, 2137, 2141, 2143, 2153, 2161, 2179, 2203, 2207, 2213, &
        2221, 2237, 2239, 2243, 2251, 2267, 2269, 2273, 2281, 2287, &
        2293, 2297, 2309, 2311, 2333, 2339, 2341, 2347, 2351, 2357, &
        2371, 2377, 2381, 2383, 2389, 2393, 2399, 2411, 2417, 2423, &
        2437, 2441, 2447, 2459, 2467, 2473, 2477, 2503, 2521, 2531, &
        2539, 2543, 2549, 2551, 2557, 2579, 2591, 2593, 2609, 2617, &
        2621, 2633, 2647, 2657, 2659, 2663, 2671, 2677, 2683, 2687, &
        2689, 2693, 2699, 2707, 2711, 2713, 2719, 2729, 2731, 2741, &
        2749, 2753, 2767, 2777, 2789, 2791, 2797, 2801, 2803, 2819, &
        2833, 2837, 2843, 2851, 2857, 2861, 2879, 2887, 2897, 2903, &
        2909, 2917, 2927, 2939, 2953, 2957, 2963, 2969, 2971, 2999, &
        3001, 3011, 3019, 3023, 3037, 3041, 3049, 3061, 3067, 3079, &
        3083, 3089, 3109, 3119, 3121, 3137, 3163, 3167, 3169, 3181, &
        3187, 3191, 3203, 3209, 3217, 3221, 3229, 3251, 3253, 3257, &
        3259, 3271, 3299, 3301, 3307, 3313, 3319, 3323, 3329, 3331, &
        3343, 3347, 3359, 3361, 3371, 3373, 3389, 3391, 3407, 3413, &
        3433, 3449, 3457, 3461, 3463, 3467, 3469, 3491, 3499, 3511, &
        3517, 3527, 3529, 3533, 3539, 3541, 3547, 3557, 3559, 3571, &
        3581, 3583, 3593, 3607, 3613, 3617, 3623, 3631, 3637, 3643, &
        3659, 3671, 3673, 3677, 3691, 3697, 3701, 3709, 3719, 3727, &
        3733, 3739, 3761, 3767, 3769, 3779, 3793, 3797, 3803, 3821, &
        3823, 3833, 3847, 3851, 3853, 3863, 3877, 3881, 3889, 3907, &
        3911, 3917, 3919, 3923, 3929, 3931, 3943, 3947, 3967, 3989, &
        4001, 4003, 4007, 4013, 4019, 4021, 4027, 4049, 4051, 4057, &
        4073, 4079, 4091, 4093, 4099, 4111, 4127, 4129, 4133, 4139, &
        4153, 4157, 4159, 4177, 4201, 4211, 4217, 4219, 4229, 4231, &
        4241, 4243, 4253, 4259, 4261, 4271, 4273, 4283, 4289, 4297, &
        4327, 4337, 4339, 4349, 4357, 4363, 4373, 4391, 4397, 4409, &
        4421, 4423, 4441, 4447, 4451, 4457, 4463, 4481, 4483, 4493, &
        4507, 4513, 4517, 4519, 4523, 4547, 4549, 4561, 4567, 4583, &
        4591, 4597, 4603, 4621, 4637, 4639, 4643, 4649, 4651, 4657, &
        4663, 4673, 4679, 4691, 4703, 4721, 4723, 4729, 4733, 4751, &
        4759, 4783, 4787, 4789, 4793, 4799, 4801, 4813, 4817, 4831, &
        4861, 4871, 4877, 4889, 4903, 4909, 4919, 4931, 4933, 4937, &
        4943, 4951, 4957, 4967, 4969, 4973, 4987, 4993, 4999, 5003, &
        5009, 5011, 5021, 5023, 5039, 5051, 5059, 5077, 5081, 5087, &
        5099, 5101, 5107, 5113, 5119, 5147, 5153, 5167, 5171, 5179, &
        5189, 5197, 5209, 5227, 5231, 5233, 5237, 5261, 5273, 5279, &
        5281, 5297, 5303, 5309, 5323, 5333, 5347, 5351, 5381, 5387, &
        5393, 5399, 5407, 5413, 5417, 5419, 5431, 5437, 5441, 5443, &
        5449, 5471, 5477, 5479, 5483, 5501, 5503, 5507, 5519, 5521, &
        5527, 5531, 5557, 5563, 5569, 5573, 5581, 5591, 5623, 5639, &
        5641, 5647, 5651, 5653, 5657, 5659, 5669, 5683, 5689, 5693, &
        5701, 5711, 5717, 5737, 5741, 5743, 5749, 5779, 5783, 5791, &
        5801, 5807, 5813, 5821, 5827, 5839, 5843, 5849, 5851, 5857, &
        5861, 5867, 5869, 5879, 5881, 5897, 5903, 5923, 5927, 5939, &
        5953, 5981, 5987, 6007, 6011, 6029, 6037, 6043, 6047, 6053, &
        6067, 6073, 6079, 6089, 6091, 6101, 6113, 6121, 6131, 6133, &
        6143, 6151, 6163, 6173, 6197, 6199, 6203, 6211, 6217, 6221, &
        6229, 6247, 6257, 6263, 6269, 6271, 6277, 6287, 6299, 6301, &
        6311, 6317, 6323, 6329, 6337, 6343, 6353, 6359, 6361, 6367, &
        6373, 6379, 6389, 6397, 6421, 6427, 6449, 6451, 6469, 6473, &
        6481, 6491, 6521, 6529, 6547, 6551, 6553, 6563, 6569, 6571, &
        6577, 6581, 6599, 6607, 6619, 6637, 6653, 6659, 6661, 6673, &
        6679, 6689, 6691, 6701, 6703, 6709, 6719, 6733, 6737, 6761, &
        6763, 6779, 6781, 6791, 6793, 6803, 6823, 6827, 6829, 6833, &
        6841, 6857, 6863, 6869, 6871, 6883, 6899, 6907, 6911, 6917, &
        6947, 6949, 6959, 6961, 6967, 6971, 6977, 6983, 6991, 6997, &
        7001, 7013, 7019, 7027, 7039, 7043, 7057, 7069, 7079, 7103, &
        7109, 7121, 7127, 7129, 7151, 7159, 7177, 7187, 7193, 7207, &
        7211, 7213, 7219, 7229, 7237, 7243, 7247, 7253, 7283, 7297, &
        7307, 7309, 7321, 7331, 7333, 7349, 7351, 7369, 7393, 7411, &
        7417, 7433, 7451, 7457, 7459, 7477, 7481, 7487, 7489, 7499, &
        7507, 7517, 7523, 7529, 7537, 7541, 7547, 7549, 7559, 7561, &
        7573, 7577, 7583, 7589, 7591, 7603, 7607, 7621, 7639, 7643, &
        7649, 7669, 7673, 7681, 7687, 7691, 7699, 7703, 7717, 7723, &
        7727, 7741, 7753, 7757, 7759, 7789, 7793, 7817, 7823, 7829, &
        7841, 7853, 7867, 7873, 7877, 7879, 7883, 7901, 7907, 7919 /)
!
      C = RESHAPE( (/  C01, C02, C03, C04, C05, C06, C07, C08, C09, C10,     &
                       C11, C12, C13, C14, C15, C16, C17, C18, C19, C20,     &
                       C21, C22, C23, C24, C25, C26, C27, C28 /),            & 
                       (/ PL, NM /),  ORDER = (/ 2, 1 /) )
!
      INFORM = 1
      INTVLS = 0
      IF ( MINVLS >= 0 ) THEN
         FINEST = 0
         VAREST = 0
         SAMPLS = MINSMP 
         DO I = 1, PL
            NP = I
            IF ( MINVLS < 2*SAMPLS*P(I) ) EXIT
         END DO
         IF ( MINVLS >= 2*SAMPLS*P(PL) ) SAMPLS = MINVLS/( 2*P(PL) ) 
      ENDIF
      DO
         VK(1) = ONE/P(NP)
         DO I = 2, N
            IF ( I <= NMP ) THEN
               VK(I) = MODULO( C(NP,N-1)*VK(I-1), ONE )
            ELSE
               VK(I) = MODULO( SQRT( REAL( PRIME(I-NMP), STND ) ), ONE )
            END IF
!            VK(I) = MODULO( 17797*VK(I-1), ONE )
         END DO

         FINVAL = 0
         VARSQR = 0
         DO I = 1, SAMPLS
            DIFINT = ( MVKRSV( N, P(NP), VK, NF, FUNCTN ) - FINVAL )/I
            FINVAL = FINVAL + DIFINT
            VARSQR = ( I - 2 )*VARSQR/I + DIFINT**2
         END DO
         INTVLS = INTVLS + 2*SAMPLS*P(NP)
         KMX = 1
         DO K = 1, NF
            VARPRD = VAREST(K)*VARSQR(K)
            FINEST(K) = FINEST(K) + ( FINVAL(K) - FINEST(K) )/( 1 + VARPRD )
            IF ( VARSQR(K) > 0 ) VAREST(K) = ( 1 + VARPRD )/VARSQR(K)
            IF ( ABS(FINEST(K)) > ABS(FINEST(KMX)) ) KMX = K
         END DO
         ABSERR = 7*SQRT( VARSQR(KMX)/( 1 + VARPRD ) )/2 
         IF ( ABSERR > MAX( ABSEPS, ABS(FINEST(KMX))*RELEPS ) ) THEN
            IF ( NP < PL ) THEN
               NP = NP + 1
            ELSE
               SAMPLS = MIN( 3*SAMPLS/2, ( MAXVLS - INTVLS )/( 2*P(NP) ) ) 
               SAMPLS = MAX( MINSMP, SAMPLS )
            ENDIF
            IF ( INTVLS + 2*SAMPLS*P(NP) > MAXVLS ) EXIT
         ELSE
            INFORM = 0
            EXIT
         ENDIF
      END DO
!
      END SUBROUTINE MVKBRV
!
      FUNCTION MVKRSV( N, PRIME, VK, NF, FUNCTN ) RESULT(VALUES)
!
! Global Variables
!
      INTEGER,                        INTENT(IN)      :: N, NF, PRIME 
      REAL(KIND=STND),  DIMENSION(:), INTENT(INOUT)   :: VK
      INTERFACE 
         FUNCTION FUNCTN( NF, X ) RESULT(VALUE)
            USE PRECISION_MODEL
            INTEGER,                       INTENT(IN) :: NF
            REAL(KIND=STND), DIMENSION(:), INTENT(IN) :: X
            REAL(KIND=STND),            DIMENSION(NF) :: VALUE
         END FUNCTION FUNCTN
      END INTERFACE   
      REAL(KIND=STND),                  DIMENSION(NF) :: VALUES
!
! Local Variables  
!
      INTEGER                       :: K
      REAL(KIND=STND), DIMENSION(N) :: X, R
!
      VALUES = 0
      DO K = 1, N
         R(K) = UNIFRM()
      END DO
      DO K = 1, PRIME
         R = R + VK
         WHERE ( R > 1 ) R = R - 1
         X = ABS ( 2*R - 1 )
         VALUES = VALUES + ( ( FUNCTN(NF,X) + FUNCTN(NF,1-X) )/2 - VALUES )/K
      END DO
!
      END FUNCTION MVKRSV
!
      FUNCTION UNIFRM() RESULT(UNI) 
!
!     Uniform (0,1) random number generator
!
!     Reference:
!     L'Ecuyer, Pierre (1996), 
!     "Combined Multiple Recursive Random Number Generators"
!     Operations Research 44, pp. 816-822.
!
      REAL(KIND=STND) :: UNI
!
      INTEGER         :: Z, H, P12, P13, P21, P23
!
!     Some eight digit primes for seeds
!
      INTEGER,   SAVE :: X10 = 15485857, X11 = 17329489, X12 = 36312197, & 
                         X20 = 55911127, X21 = 75906931, X22 = 96210113       
      REAL(KIND=STND), PARAMETER :: INVMP1 = 4.656612873077392578125E-10_STND 
!                                   INVMP1 = 1/(M1+1)
      INTEGER, PARAMETER :: M1 = 2147483647, M2 = 2145483479 
      INTEGER, PARAMETER :: A12 =   63308, Q12 = 33921, R12 = 12979 
      INTEGER, PARAMETER :: A13 = -183326, Q13 = 11714, R13 =  2883 
      INTEGER, PARAMETER :: A21 =   86098, Q21 = 24919, R21 =  7417 
      INTEGER, PARAMETER :: A23 = -539608, Q23 =  3976, R23 =  2071 
!
!
!     Component 1
!
      H = X10/Q13
      P13 = -A13*( X10 - H*Q13 ) - H*R13
      H = X11/Q12
      P12 =  A12*( X11 - H*Q12 ) - H*R12
      IF ( P13 < 0 ) THEN
         P13 = P13 + M1
      END IF
      IF ( P12 < 0 ) THEN
         P12 = P12 + M1
      END IF
      X10 = X11 
      X11 = X12
      X12 = P12 - P13
      IF ( X12 < 0 ) THEN
         X12 = X12 + M1
      END IF
!
!     Component 2
!
      H = X20/Q23
      P23 = -A23*( X20 - H*Q23 ) - H*R23
      H = X22/Q21
      P21 =  A21*( X22 - H*Q21 ) - H*R21
      IF ( P23 < 0 ) THEN
         P23 = P23 + M2
      END IF
      IF ( P21 < 0 ) THEN
         P21 = P21 + M2
      END IF
      X20 = X21 
      X21 = X22
      X22 = P21 - P23
      IF ( X22 < 0 ) THEN
         X22 = X22 + M2
      END IF
!
!     Combination
!
      Z = X12 - X22
      IF ( Z <= 0 ) THEN
         Z = Z + M1
      END IF
      UNI = Z*INVMP1
!
      END FUNCTION UNIFRM
END MODULE MVBRVMOD
